#include "kups_printerinstances.h"

#include <qlistview.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <kiconloader.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <klocale.h>

#include "kinstancemanager.h"
#include "kprinterinstance.h"
#include "kups_printeritem.h"
#include "kprinter.h"
#include "qinputbox.h"

extern void stopTimer();
extern void startTimer();
extern void triggerUpdate();

Kups_PrinterInstances::Kups_PrinterInstances(QWidget *parent, const char *name)
: QWidget(parent, name)
{
	list_ = new QListBox(this);

	add_ = new QPushButton(i18n("Add"), this);
	copy_ = new QPushButton(i18n("Copy"), this);
	default_ = new QPushButton(i18n("Set as default"), this);
	edit_ = new QPushButton(i18n("Settings..."), this);
	remove_ = new QPushButton(i18n("Remove"), this);
	test_ = new QPushButton(i18n("Test"), this);
	connect(add_, SIGNAL(clicked()), SLOT(slotAdd()));
	connect(copy_, SIGNAL(clicked()), SLOT(slotCopy()));
	connect(edit_, SIGNAL(clicked()), SLOT(slotEdit()));
	connect(default_, SIGNAL(clicked()), SLOT(slotDefault()));
	connect(remove_, SIGNAL(clicked()), SLOT(slotRemove()));
	connect(test_, SIGNAL(clicked()), SLOT(slotTest()));

	QHBoxLayout	*mainLayout_ = new QHBoxLayout(this, 15, 20);
	QVBoxLayout	*btnLayout_ = new QVBoxLayout(0, 0, 15);

	mainLayout_->addWidget(list_, 1);
	mainLayout_->addLayout(btnLayout_, 0);
	btnLayout_->addWidget(add_, 0);
	btnLayout_->addWidget(copy_, 0);
	btnLayout_->addWidget(default_, 0);
	btnLayout_->addWidget(edit_, 0);
	btnLayout_->addWidget(remove_, 0);
	btnLayout_->addWidget(test_, 0);
	btnLayout_->addStretch(1);

	slotPrinterSelected(0);
}

Kups_PrinterInstances::~Kups_PrinterInstances()
{
}

void Kups_PrinterInstances::slotPrinterSelected(QListViewItem *item)
{
	KPrinter	*pr(0);
	if (item && item->depth() >= 2)
	{
		pr = ((Kups_PrinterItem*)item)->getPrinter();
	}
	updateForPrinter(pr);
}

void Kups_PrinterInstances::updateForPrinter(KPrinter *printer)
{
	QString	oldText = list_->currentText();
	KPrinter	*oldPrinter = printer_;
	bool	on = (printer != 0);

	printer_ = printer;
	list_->clear();
	add_->setEnabled(on);
	edit_->setEnabled(on);
	remove_->setEnabled(on);
	default_->setEnabled(on);
	copy_->setEnabled(on);
	test_->setEnabled(on);
	if (printer_)
	{
		QList<KPrinterInstance>	list = KInstanceManager::get()->getInstancesForPrinter(printer_->getName());
		list.setAutoDelete(false);
		for (list.first();list.current();list.next())
			list_->insertItem((list.current()->isDefault() ? SmallIcon("exec") : SmallIcon("fileprint")), list.current()->getInstance());
	}

	QListBoxItem	*litem;
	if (printer_ == oldPrinter && (litem=list_->findItem(oldText)))
	{
		list_->setSelected(litem, true);
	}
}

void Kups_PrinterInstances::slotAdd()
{
	::stopTimer();

	bool	ok(true);
	QString	name = QInputBox::inputBox(this, i18n("Add instance"), i18n("Enter name for new instance (leave blank for default instance)"), QString::null, &ok);
	if (ok && printer_)
	{
		if (name.isEmpty()) name = i18n("(Default)");
		if (list_->findItem(name) != 0)
		{
			QMessageBox::critical(this, i18n("Add instance"), i18n("The instance \"%1\" already exists !").arg(name), QMessageBox::Ok|QMessageBox::Default, 0);
		}
		else
		{
			if (!KInstanceManager::get()->createInstance(printer_->getName(), name))
			{
				QMessageBox::critical(this, i18n("Add instance"), KInstanceManager::get()->getLastError(), QMessageBox::Ok|QMessageBox::Default, 0);
			}
			else
			{
				updateForPrinter(printer_);
			}
		}
	}

	::startTimer();
}

void Kups_PrinterInstances::slotCopy()
{
	::stopTimer();

	if (!list_->currentText().isEmpty() && printer_)
	{
		bool	ok(true);
		QString	name = QInputBox::inputBox(this, i18n("Copy instance"), i18n("Enter name for new instance"), list_->currentText(), &ok);
		if (ok && !name.isEmpty())
		{
			if (list_->findItem(name) != 0)
			{
				QMessageBox::critical(this, i18n("Copy instance"), i18n("The instance \"%1\" already exists !").arg(name), QMessageBox::Ok|QMessageBox::Default, 0);
			}
			else
			{
				if (!KInstanceManager::get()->copyInstance(printer_->getName(), list_->currentText(), name))
				{
					QMessageBox::critical(this, i18n("Copy instance"), KInstanceManager::get()->getLastError(), QMessageBox::Ok|QMessageBox::Default, 0);
				}
				else
				{
					updateForPrinter(printer_);;
				}
			}
		}
		else if (ok && name.isEmpty())
		{
			QMessageBox::critical(this, i18n("Copy instance"), i18n("Invalid (empty) instance name !"), QMessageBox::Ok|QMessageBox::Default, 0);
		}
	}

	::startTimer();
}

void Kups_PrinterInstances::slotEdit()
{
	::stopTimer();

	if (!list_->currentText().isEmpty() && printer_)
	{
		if(!KInstanceManager::get()->editInstance(printer_->getName(), list_->currentText()))
		{
			QMessageBox::critical(this, i18n("Edit instance"), KInstanceManager::get()->getLastError(), QMessageBox::Ok|QMessageBox::Default, 0);
		}
	}

	::startTimer();
}

void Kups_PrinterInstances::slotRemove()
{
	::stopTimer();

	if (!list_->currentText().isEmpty() && printer_)
	{
		if (QMessageBox::warning(this, i18n("Remove instance"), i18n("Are you sure you want to remove the instance \"%1\" ?").arg(list_->currentText()), QMessageBox::Yes|QMessageBox::Default, QMessageBox::No) == QMessageBox::Yes)
		{
			if(!KInstanceManager::get()->removeInstance(printer_->getName(), list_->currentText()))
			{
				QMessageBox::critical(this, i18n("Remove instance"), KInstanceManager::get()->getLastError(), QMessageBox::Ok|QMessageBox::Default, 0);
			}
			else
			{
				updateForPrinter(printer_);
			}
		}
	}

	::startTimer();
}

void Kups_PrinterInstances::slotDefault()
{
	::stopTimer();

	if (!list_->currentText().isEmpty() && printer_)
	{
		if(!KInstanceManager::get()->setDefaultInstance(printer_->getName(), list_->currentText()))
		{
			QMessageBox::critical(this, i18n("Default instance"), KInstanceManager::get()->getLastError(), QMessageBox::Ok|QMessageBox::Default, 0);
		}
		else
		{
			updateForPrinter(printer_);
		}
	}

	::startTimer();
}

void Kups_PrinterInstances::slotTest()
{
	::stopTimer();

	if (!list_->currentText().isEmpty() && printer_)
	{
		if (!KInstanceManager::get()->testInstance(printer_->getName(), list_->currentText()))
		{
			QMessageBox::critical(this, i18n("Test instance"), KInstanceManager::get()->getLastError(), QMessageBox::Ok|QMessageBox::Default, 0);
		}
		else
		{
			QMessageBox::information(this, i18n("Test instance"), i18n("Test page has been successfully send to the printer \"%1\".").arg(printer_->getName()), QMessageBox::Ok|QMessageBox::Default, 0);
		}
	}

	::startTimer();
}
